@@ -4,8 +4,10 @@ import abc
 import asyncio
 from collections import defaultdict
 import functools
+import hashlib
 import logging
 import secrets
+import socket
 from typing import (
     Any,
     Callable,
@@ -29,6 +31,7 @@ import aioredis
 import aioredis.exceptions
 import aioredis.sentinel
 from aiotools.context import aclosing
+from aiotools.server import process_index
 import attr
 
 from . import msgpack, redis
@@ -493,9 +496,9 @@ class RedisConnectorFunc(Protocol):
         ...
 
 
-TEvent = TypeVar('TEvent', bound='AbstractEvent', contravariant=True)
+TEvent = TypeVar('TEvent', bound='AbstractEvent')
 TEventCov = TypeVar('TEventCov', bound='AbstractEvent')
-TContext = TypeVar('TContext', contravariant=True)
+TContext = TypeVar('TContext')
 
 EventCallback = Union[
     Callable[[TContext, AgentId, TEvent], Coroutine[Any, Any, None]],
@@ -553,7 +556,8 @@ class CoalescingState:
         )
         if self.last_added > 0 and loop.time() - self.last_added < opts['max_wait']:
             # Cancel the previously pending task.
-            self.last_handle.cancel()
+            if not self.last_handle.cancelled():
+                self.last_handle.cancel()
             self.fut_sync.cancel()
             # Reschedule.
             self.fut_sync = loop.create_future()
@@ -564,6 +568,8 @@ class CoalescingState:
         try:
             await self.fut_sync
         except asyncio.CancelledError:
+            if self.last_handle is not None and not self.last_handle.cancelled():
+                self.last_handle.cancel()
             return False
         else:
             self.fut_sync = None
@@ -606,13 +612,14 @@ class EventDispatcher(aobject):
         log_events: bool = False,
         *,
         consumer_group: str = "manager",
+        node_id: str = None,
     ) -> None:
         self.redis_client = redis.get_redis_object(connector, db=db)
         self._log_events = log_events
         self.consumers = defaultdict(set)
         self.subscribers = defaultdict(set)
         self._consumer_group = consumer_group
-        self._consumer_name = secrets.token_urlsafe(16)
+        self._consumer_name = _generate_consumer_id(node_id)
 
     async def __ainit__(self) -> None:
         self.consumer_loop_task = asyncio.create_task(self._consume_loop())
@@ -621,21 +628,26 @@ class EventDispatcher(aobject):
         self.subscriber_taskset = weakref.WeakSet()
 
     async def close(self) -> None:
-        cancelled_tasks = []
-        for task in self.consumer_taskset:
-            if not task.done():
-                task.cancel()
-                cancelled_tasks.append(task)
-        for task in self.subscriber_taskset:
-            if not task.done():
-                task.cancel()
-                cancelled_tasks.append(task)
-        self.consumer_loop_task.cancel()
-        self.subscriber_loop_task.cancel()
-        cancelled_tasks.append(self.consumer_loop_task)
-        cancelled_tasks.append(self.subscriber_loop_task)
-        await asyncio.gather(*cancelled_tasks, return_exceptions=True)
-        await self.redis_client.close()
+        try:
+            cancelled_tasks = []
+            for task in self.consumer_taskset:
+                if not task.done():
+                    task.cancel()
+                    cancelled_tasks.append(task)
+            for task in self.subscriber_taskset:
+                if not task.done():
+                    task.cancel()
+                    cancelled_tasks.append(task)
+            await asyncio.sleep(0)
+            self.consumer_loop_task.cancel()
+            self.subscriber_loop_task.cancel()
+            cancelled_tasks.append(self.consumer_loop_task)
+            cancelled_tasks.append(self.subscriber_loop_task)
+            await asyncio.gather(*cancelled_tasks, return_exceptions=True)
+        except Exception:
+            log.exception("unexpected error while closing event dispatcher")
+        finally:
+            await self.redis_client.close()
 
     def consume(
         self,
@@ -800,3 +812,14 @@ class EventProducer(aobject):
             self.redis_client,
             lambda r: r.xadd('events', raw_event),  # type: ignore # aio-libs/aioredis-py#1182
         )
+
+
+def _generate_consumer_id(node_id: str = None) -> str:
+    h = hashlib.sha1()
+    h.update(str(node_id or socket.getfqdn()).encode('utf8'))
+    hostname_hash = h.hexdigest()
+    h = hashlib.sha1()
+    h.update(__file__.encode('utf8'))
+    installation_path_hash = h.hexdigest()
+    pidx = process_index.get(0)
+    return f"{hostname_hash}:{installation_path_hash}:{pidx}"
